@@ -1,5 +1,10 @@
 # -*- coding: utf-8 -*-
 
+import logging
+from numbers import Number
+from collections import OrderedDict, UserList
+import datetime
+import array
 from .serializable import Serializable
 from .datatypes import DataTypes, DataTypeNames
 from .odict import ODict
@@ -8,25 +13,20 @@ from .listener import DatasetEventSender, ParameterListener, DatasetEvent, Event
 from .eq import DeepEqual, xhash
 from .copyable import Copyable
 from .annotatable import Annotatable
-from .classes import Classes
+# from .classes import Classes, Class_Module_Map
+from . import classes  # import Classes, _bltn
 from .typed import Typed
 from .invalid import INVALID
 from ..utils.masked import masked
 from ..utils.common import grouper
 from ..utils.common import exprstrs, wls, bstr, t2l
-from fdi.dataset.listener import ListnerSet
+from fdi.dataset.listener import ListenerSet
 
 import cwcwidth as wcwidth
 import tabulate
 
-from itertools import zip_longest, filterfalse
-import builtins
-import array
-import datetime
 import copy
-from collections import OrderedDict, UserList
-from numbers import Number
-import logging
+import builtins
 # create logger
 logger = logging.getLogger(__name__)
 # logger.debug('level %d' %  (logger.getEffectiveLevel()))
@@ -363,6 +363,7 @@ class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Se
         return self.__class__.__name__ + ss
 
     string = toString
+    txt = toString
 
     def __getstate__(self):
         """ Can be encoded with serializableEncoder
@@ -382,6 +383,8 @@ class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Se
 def guess_value(data, parameter=False, last=str):
     """ Returns guessed value from a string.
 
+    This is different from `Attributable.value2parameter`
+
     | input | output |
     | ```'None'```,```'null'```,```'nul'``` any case | `None` |
     | integer | `int()` |
@@ -390,6 +393,10 @@ def guess_value(data, parameter=False, last=str):
     | string starting with ```'0x'``` | `hex()` |
     | else | run `last`(data) |
 
+    Returns
+    -------
+    Parameter
+
     """
     from .numericparameter import NumericParameter, BooleanParameter
     from .dateparameter import DateParameter
@@ -399,54 +406,54 @@ def guess_value(data, parameter=False, last=str):
     from .finetime import FineTime
     if data is None:
         return Parameter(value=data) if parameter else data
-    else:
-        if issubclass(data.__class__, (list, tuple, set, array.array)):
+
+    if issubclass(data.__class__, (list, tuple, set, array.array)):
+        res = data
+        return NumericParameter(value=res) if parameter else res
+    try:
+        if issubclass(data.__class__, int):
             res = data
-            return NumericParameter(value=res) if parameter else res
+        else:
+            res = int(data)
+        return NumericParameter(value=res) if parameter else res
+    except (ValueError, TypeError):
         try:
-            if issubclass(data.__class__, int):
+            if issubclass(data.__class__, float):
                 res = data
             else:
-                res = int(data)
+                res = float(data)
             return NumericParameter(value=res) if parameter else res
         except (ValueError, TypeError):
-            try:
-                if issubclass(data.__class__, float):
-                    res = data
-                else:
-                    res = float(data)
+            # string, bytes, bool
+            if issubclass(data.__class__, bytes):
+                res = data
                 return NumericParameter(value=res) if parameter else res
-            except (ValueError, TypeError):
-                # string, bytes, bool
-                if issubclass(data.__class__, bytes):
-                    res = data
-                    return NumericParameter(value=res) if parameter else res
-                if issubclass(data.__class__, bool):
-                    res = data
-                    return BooleanParameter(value=res) if parameter else res
-                if issubclass(data.__class__, (datetime.datetime, FineTime)):
-                    res = data
-                    return DateParameter(value=res) if parameter else res
-                elif data[:4].upper() in ('NONE', 'NULL', 'NUL'):
-                    return Parameter(value=None) if parameter else None
-                elif data.startswith('0x'):
-                    res = bytes.fromhex(data[2:])
-                    return NumericParameter(value=res) if parameter else res
-                elif data.upper() in ['TRUE', 'FALSE']:
-                    res = bool(data)
-                    return BooleanParameter(value=res) if parameter else res
-                elif len(data) > 16 and data[0] in '0987654321' and 'T' in data and ':' in data and '-' in data:
-                    res = FineTime(data)
-                    return DateParameter(value=res) if parameter else res
-                else:
-                    res = last(data)
-                    return Parameter(value=res) if parameter else res
+            if issubclass(data.__class__, bool):
+                res = data
+                return BooleanParameter(value=res) if parameter else res
+            if issubclass(data.__class__, (datetime.datetime, FineTime)):
+                res = data
+                return DateParameter(value=res) if parameter else res
+            elif data[:4].upper() in ('NONE', 'NULL', 'NUL'):
+                return Parameter(value=None) if parameter else None
+            elif data.startswith('0x'):
+                res = bytes.fromhex(data[2:])
+                return NumericParameter(value=res) if parameter else res
+            elif data.upper() in ['TRUE', 'FALSE']:
+                res = bool(data)
+                return BooleanParameter(value=res) if parameter else res
+            elif len(data) > 16 and data[0] in '0987654321' and 'T' in data and ':' in data and '-' in data:
+                res = FineTime(data)
+                return DateParameter(value=res) if parameter else res
+            else:
+                res = last(data)
+                return Parameter(value=res) if parameter else res
     return StringParameter('null') if parameter else None
 
 
 def make_jsonable(valid):
 
-    return[t2l([k, v]) for k, v in valid.items()] if issubclass(valid.__class__, dict) else t2l(valid)
+    return [t2l([k, v]) for k, v in valid.items()] if issubclass(valid.__class__, dict) else t2l(valid)
 
 
 Seqs = (list, tuple, UserList)
@@ -454,7 +461,9 @@ Seqs = (list, tuple, UserList)
 
 class Parameter(AbstractParameter, Typed):
     """ Parameter is the interface for all named attributes
-    in the MetaData container. It can have a value and a description.
+    in the MetaData container.
+
+     It can have a value and a description.
     Default arguments: typ_='', default=None, valid=None.
     value=default, description='UNKNOWN'
     """
@@ -483,11 +492,12 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
         """
 
         # collect args-turned-local-variables.
-        args = OrderedDict(filterfalse(
-            lambda x: x[0] in ('self', '__class__', 'kwds'),
-            locals().items())
-        )
+        args = copy.copy(locals())
+        args.pop('__class__', None)
+        args.pop('kwds', None)
+        args.pop('self', None)
         args.update(kwds)
+
         self._all_attrs = args
 
         self.setDefault(default)
@@ -512,11 +522,13 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
 
         Default will be casted if not the same.
         Unsuported parameter types will get a NotImplementedError.
+
         Parameters
         ----------
 
         Returns
         -------
+        None
 
         """
         if typ_ is None or typ_ == '':
@@ -531,6 +543,9 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
                 'Parameter type %s is not in %s.' %
                 (typ_, str([''.join(x) for x in DataTypes])))
 
+    ALLOWED_PARAM_DATA_TYPES = DataTypeNames
+    ALLOWED_PARAM_DATA_TYPES.update({'array': 'array'})
+
     def checked(self, value):
         """ Checks input value against self.type.
 
@@ -538,6 +553,7 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
         else if type is not set, return value after setting type;
         If value's type is a subclass of self's type, return the value;
         If value's and self's types are both subclass of Number, returns value casted in self's type.
+
         Parameters
         ----------
 
@@ -546,49 +562,69 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
 
         """
         if not hasattr(self, '_type'):
-
             return value
 
-        t_type = type(value)
-        t = t_type.__name__
-        st = self._type
-        if st == '' or st is None:
+        value_class = type(value)
+        value_cls_name = value_class.__name__
+        self_type = self._type
+        if self_type == '' or self_type is None:
             # self does not have a type
             try:
-                ct = DataTypeNames[t]
-                if ct == 'vector':
+                ct = self.ALLOWED_PARAM_DATA_TYPES[value_cls_name]
+                if 0 and ct == 'vector':
                     self._type = 'quaternion' if len(value) == 4 else ct
                 else:
                     self._type = ct
             except KeyError as e:
-                raise TypeError("Type %s is not in %s." %
-                                (t, str([''.join(x) for x in DataTypeNames])))
+                if 0:
+                    raise TypeError("Type %s is not in %s." %
+                                    (value_cls_name,
+                                     str([''.join(x) for x in
+                                          self.ALLOWED_PARAM_DATA_TYPES])))
+                self._type = None
             return value
 
-        # self has type
-        tt = DataTypes[st]
-        if tt in Classes.mapping:
+        self_cls_name = DataTypes[self_type]
+        if 0:
+            logger1 = str(value)
+            logger2 = self_cls_name+'+++ %x %d' % \
+                (id(classes._bltn), len(classes.Classes.mapping.maps[2]))
+
+        if self_cls_name not in classes.Classes.mapping:
+            # __import__('pdb').set_trace()
+            if 0:
+                logger.warning(logger1+logger2 + str(value))
+                logger.warning(self_cls_name+'+++$$$$$$ %x %d' %
+                               (id(classes._bltn), len(classes.Classes.mapping.maps[2])))
+
+            return value
+        if self_cls_name in classes.Class_Module_Map:
             # custom-defined parameter. delegate checking to themselves
-            if issubclass(type(value), tuple):
-                # frozendict used in baseproduct module change lists to tuples
-                # which causes deserialized parameter to differ from ProductInfo.
-                value = list(value)
             return value
-        tt_type = builtins.__dict__[tt]
-        if issubclass(t_type, tt_type):
+
+        if issubclass(type(value), tuple):
+            # frozendict used in baseproduct module change lists to tuples
+            # which causes deserialized parameter to differ from ProductInfo.
+            value = list(value)
+        self_class = classes.Classes.mapping[self_cls_name]
+        # if value type is a subclass of self type
+        # if issubclass(value_class, float):
+
+        if issubclass(value_class, self_class):
             return value
-        elif issubclass(t_type, Number) and issubclass(tt_type, Number):
+        elif issubclass(value_class, Number) and issubclass(self_class, Number):
             # , if both are Numbers.Number, value is casted into given typ_.
-            return tt_type(value)
-            # st = tt
-        elif issubclass(t_type, Seqs) and issubclass(tt_type, Seqs):
+            return self_class(value)
+            # self_type = self_cls_name
+        elif issubclass(value_class, Seqs) and issubclass(self_class, Seqs):
             # , if both are Numbers.Number, value is casted into given typ_.
-            return tt_type(value)
-            # st = tt
+            return self_class(value)
+            # self_type = self_cls_name
         else:
-            vs = hex(value) if t == 'int' and st == 'hex' else str(value)
+            vs = hex(value) if value_cls_name == 'int' and self_type == 'hex' else str(
+                value)
             raise TypeError(
-                'Value %s is of type %s, but should be %s.' % (vs, t, tt))
+                'Value %s is of type %s, but should be %s.' % (vs, value_cls_name, self_cls_name))
 
     def setValue(self, value):
         """ Replaces the current value of this parameter.
@@ -842,6 +878,7 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
             (self.__class__.__name__, ts, vs, us, ds, fs, gs, cs)
 
     string = toString
+    txt = toString
 
     __str__ = toString
 
@@ -888,7 +925,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
 
     MaxDefWidth = max(Default_Param_Widths.values())
 
-    def __init__(self, copy=None, defaults=None, **kwds):
+    def __init__(self, copy_=None, defaults=None, **kwds):
         """
 
         Parameters
@@ -899,9 +936,9 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
         """
 
         super().__init__(**kwds)
-        if copy:
+        if copy_:
             # not implemented ref https://stackoverflow.com/questions/10640642/is-there-a-decent-way-of-creating-a-copy-constructor-in-python
-            raise ValueError('use copy.copy() insteadof MetaData(copy)')
+            raise ValueError('use copy.copy() insteadof MetaData(copy_)')
         else:
             self._defaults = [] if defaults is None else defaults
             return
@@ -990,16 +1027,25 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
             self.fire(e)
         return r
 
-    def toString(self, level=0,
-                 tablefmt='grid', tablefmt1='simple', tablefmt2='rst',
-                 extra=False, param_widths=None, **kwds):
+    def toString(self, level=0, extra=False, param_widths=None,
+                 tablefmt='grid', tablefmt1='simple', tablefmt2='psql',
+                 **kwds):
         """ return  string representation of metada.
 
-        level: 0 is the most detailed, 2 is the least,
-        tablefmt: format string in packae ``tabulate``, for level==0, tablefmt1 for level1, tablefmt2: format of 2D table data.
-        param_widths: controls how the attributes of every parameter are displayed in the table cells. If is set to -1, there is no cell-width limit. For finer control set a dictionary of parameter attitute names and how many characters wide its table cell is, 0 for ommiting the attributable. Default is `MetaData.Default_Param_Widths`. e.g.
-``{'name': 15, 'value': 18, 'unit': 6, 'type': 8,
-         'valid': 17, 'default': 15, 'code': 4, 'description': 17}``
+        The order of parameters are important as the httppool API use the order when passing parameters to, e.g. `txt`.
+
+        Parameters
+        ----------
+        level : int
+            detailed-ness level. 0 is the most detailed, 2 is the least,
+        param_widths : int or dict
+            Controls how the attributes of every parameter are displayed in the table cells. If is set to -1, there is no cell-width limit (e.g. fot html output). If set to >=0, all widths are set to this. For finer control set a dictionary of parameter attribute names and how many characters wide its table cell is, 0 for ommiting the attributable. Default is `MetaData.Default_Param_Widths`. Example: ``{'name': 15, 'value': 18, 'unit': 6, 'type': 8, 'valid': 17, 'default': 15, 'code': 4, 'description': 17}``. default is
+        tablefmt : str
+            format string in packae ``tabulate``, for level==0.
+        tablefmt1 : string
+            for level1
+        tablefmt2 : str
+            format of 2D table data.
         """
 
         html = 'html' in tablefmt.lower() or 'html' in tablefmt2.lower()
@@ -1012,13 +1058,54 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
         s = ''
         att, ext = {}, {}
         has_omission = False
-        nn = 0
+
+        if param_widths is None:
+            param_widths = copy.copy(MetaData.Default_Param_Widths)
+
+        def _fix_bool(l):
+
+            # __import__("pdb").set_trace()
+
+            # work around a bug in tabulate
+            for i in range(len(l)):
+                v = l[i]
+                if isinstance(v, bool):
+                    v = str(v)
+                ll = v.lower()
+                if ll in ('true', 'false'):
+                    l[i] = ll
+            #ll = l[5].lower()
+            # if l[5] in ('true', 'false'):
+            #    l[5] = ll
+
+        def get_thewidths_l0(param_widths):
+            # limit cell width for level=0,1.
+            if isinstance(param_widths, int) and param_widths <= -1 or html:
+                thewidths = dict((n, MetaData.MaxDefWidth)
+                                 for n in MetaHeaders)
+            else:
+                # param_widths is not -1. can be 0, >0 , list
+                if isinstance(param_widths, int):
+                    if param_widths > 0:
+                        thewidths = dict((n, param_widths)
+                                         for n in MetaHeaders)
+                    else:
+                        # == 0
+                        thewidths = copy.copy(MetaData.Default_Param_Widths)
+                else:
+                    thewidths = param_widths if param_widths else \
+                        copy.copy(MetaData.Default_Param_Widths)
+            return thewidths
+
+        ext_hdr = []
         for (k, v) in self.__getstate__().items():
             if k.startswith('_ATTR_'):
                 k = k[6:]
             elif k == '_STID':
                 continue
             att['name'] = k
+            # if k == 'urls':
+            #    __import__('pdb').set_trace()
 
             # get values of line k.
             if issubclass(v.__class__, Parameter):
@@ -1029,6 +1116,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                         tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
                         extra=extra,
                         alist=True)
+
                 from ..utils.fits_kw import getFitsKw
                 # make sure every line has fits_keyword in ext #1
                 fk = ext.pop('fits_keyword') if 'fits_keyword' \
@@ -1036,13 +1124,12 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                 ext0 = ext
                 ext = {'fits_keyword': fk}
                 ext.update(ext0)
-            elif issubclass(v.__class__, ListnerSet):
+            elif issubclass(v.__class__, ListenerSet):
                 # listeners
                 lstr = '' if v is None else v.toString(level=level, alist=True)
                 if len(lstr) < 3:
                     lstr = [["", "<No listener>", ""]]
-                att['value'], att['unit'], att['type'], att['description'] = \
-                    '\n'.join(str(x[1]) for x in lstr), '', \
+                att['value'], att['unit'], att['type'], att['description'] = '\n'.join(str(x[1]) for x in lstr), '', \
                     '\n'.join(x[0] for x in lstr), \
                     '\n'.join(x[2] for x in lstr)
                 att['default'], att['valid'], att['code'] = '', '', ''
@@ -1053,41 +1140,31 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
 
             # if tablefmt == 'html':
             #    att['valid'] = att['valid'].replace('\n', '<br>')
-            # generate column vallues of the line and ext headers
-            # limit cell width for level=0,1.
+
             if level == 0:
-                if param_widths == -1 or html:
-                    w = MetaData.MaxDefWidth * 2
-                    l = tuple(wls(att[n], w, linebreak=br)
-                              for n in MetaHeaders)
-                    if extra:
-                        l += tuple(v for v in ext.values())
-                else:
-                    thewidths = param_widths if param_widths else \
-                        MetaData.Default_Param_Widths
-                    l = tuple(wls(att[n], w)
-                              for n, w in thewidths.items() if w != 0)
-                    if extra:
-                        l += tuple(
-                            wls(v, Default_Extra_Param_Width)
-                            for v in ext.values())
-                        # print(l)
+                # generate column values of the line and ext headers
+                # limit cell width for level=0,1.
+                thewidths = get_thewidths_l0(param_widths)
+                l = list(map(str, att.values()))
+                #print('+++ %s' % str(l))
+                if l[3] == 'boolean':
+                    _fix_bool(l)
+                # print('### %s' % str(l))
+                if extra:
+                    l += list(map(str, ext.values()))
+                l = tuple(l)
 
                 tab.append(l)
-                ext_hdr = [v for v in ext.keys()]
+                if 0:
+                    ext_hdr = [v for v in ext.keys()]
+                else:
+                    for v in ext.keys():
+                        if v not in ext_hdr:
+                            ext_hdr.append(v)
 
             elif level == 1:
-                ps = '%s= %s' % (att['name'], att['value'])
-                tab.append(wls(ps, 80//N))
-                # s += mstr(self, level=level, tablefmt = tablefmt, \
-                # tablefmt=tablefmt, tablefmt1=tablefmt1, \
-                # tablefmt2=tablefmt2,depth=1, **kwds)
-                if 0:
-                    row.append(wls(ps, 80//N))
-                    i += 1
-                    if i == N:
-                        tab.append(row)
-                        i, row = 0, []
+                ps = '%s= %s' % (att['name'], str(att['value']))
+                tab.append(wls(ps, 81//N))
             else:
                 # level > 1
                 n = att['name']
@@ -1102,38 +1179,27 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                     ps = '%s=%s' % (n, v.toString(level)) if level == 2 else n
                     # tab.append(wls(ps, 80//N))
                     tab.append(ps)
-            # nn += 1
-            # if nn == 2:
-            #    pass  # break
 
         if has_omission:
             tab.append('..')
 
         # write out the table
         if level == 0:
-            allh = copy.copy(MetaHeaders)
+            thewidths = get_thewidths_l0(param_widths)
+            headers = list(thewidths)
             if extra:
-                allh += ext_hdr
-            if param_widths == -1 or html:
-                headers = allh
-            else:
-                headers = []
-                thewidths = param_widths if param_widths else \
-                    MetaData.Default_Param_Widths
-                for n in allh:
-                    w = thewidths.get(n, Default_Extra_Param_Width)
-                    #print(n, w)
-                    if w != 0:
-                        headers.append(wls(n, w))
+                headers.extend(ext_hdr)
             fmt = tablefmt
-            maxwidth = MetaData.MaxDefWidth
+            maxwidth = list(thewidths.values())
             s += tabulate.tabulate(tab, headers=headers, tablefmt=fmt,
                                    missingval='', maxcolwidths=maxwidth,
                                    disable_numparse=True)
+
         elif level == 1:
             t = grouper(tab, N)
             headers = ''
             fmt = tablefmt1
+
             s += tabulate.tabulate(t, headers=headers, tablefmt=fmt, missingval='',
                                    disable_numparse=True)
         elif level > 1:  # level 2 and 3
@@ -1147,6 +1213,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
         #    '%s %s-listeners = %s' % ('(No Parameter.)', cn, lsnr)
 
     string = toString
+    txt = toString
 
     def __getstate__(self):
         """ Can be encoded with serializableEncoder
