@@ -50,7 +50,7 @@ else:
 
 
 # Global variables set to temprary values before setGlabals() runs
-logger = __import__('logging').getLogger(__name__)
+# logger = __import__('logging').getLogger(__name__)
 
 data_api = Blueprint('httppool_server', __name__)
 
@@ -68,7 +68,7 @@ def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', l
     :ctype: Content-Type. Default is `application/json`
     :serialize_out: if True `result` is in already in serialized form.
     """
-
+    logger = current_app.logger
     # return if `result` is already a Response
     if issubclass(result.__class__, Response):
         resp = result
@@ -142,7 +142,8 @@ def urn(parts):
     logger = current_app.logger
 
     ts = time.time()
-    logger.debug('get data for URN parts ' + parts)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('get data for URN parts ' + parts)
 
     paths = parts2paths(parts)
     # if paths[-1] == '':
@@ -170,7 +171,8 @@ def delete_urn(parts):
     logger = current_app.logger
 
     ts = time.time()
-    logger.info('delete data for URN parts ' + parts)
+    if logger.isEnabledFor(logging_INFO):
+        logger.info('delete data for URN parts ' + parts)
 
     paths = parts2paths(parts)
     # if paths[-1] == '':
@@ -206,6 +208,7 @@ def delete_product(paths, serialize_out=False):
     """
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
 
+    logger = current_app.logger
     typename = paths[1]
     indexstr = paths[2]
     poolname = paths[0]
@@ -217,12 +220,15 @@ def delete_product(paths, serialize_out=False):
         result = FAILED
         msg = 'Pool not found or not registered: ' + poolname
         code = 404
-        logger.error(msg)
+        if logger.isEnabledFor(logging_ERROR):
+            logger.error(msg)
         return code, result, msg
-    logger.debug('DELETE product urn: ' + urn)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('DELETE product urn: ' + urn)
     try:
         poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
-        logger.debug('**** '+str(list(poolobj._dTypes))+urn)
+        if logger.isEnabledFor(logging_DEBUG):
+            logger.debug('**** '+str(list(poolobj._dTypes))+urn)
         poolobj.remove(urn)
         result = 0
         msg = 'remove product ' + urn + ' OK.'
@@ -231,7 +237,8 @@ def delete_product(paths, serialize_out=False):
         code, result, msg = excp(
             e,
             msg='Unable to remove product: ' + urn)
-        logger.error(msg)
+        if logger.isEnabledFor(logging_ERROR):
+            logger.error(msg)
     return code, result, msg
 
 ######################################
@@ -259,7 +266,8 @@ def save_data(pool):
     if not request.data:
         result, msg = '"FAILED"', 'No Request data for command '+request.method
         code = 400
-        logger.warning(f'SAVE product to {pool} {msg}.')
+        if logger.isEnabledFor(logging_INFO):
+            logger.warning(f'SAVE product to {pool} {msg}.')
         return resp(code, result, msg, ts, serialize_out=True)
 
     # save product
@@ -285,7 +293,8 @@ def save_data(pool):
                 e,
                 msg='Class needs to be included in pool configuration.',
                 serialize_out=serial_through)
-            logger.info(f'SAVE product to {poolurl} {msg}.')
+            if logger.isEnabledFor(logging_INFO):
+                logger.info(f'SAVE product to {poolurl} {msg}.')
         else:
             code, result, msg = save_product(
                 data, paths, tags, serialize_in=not serial_through, logger=logger)
@@ -294,7 +303,7 @@ def save_data(pool):
     return resp(code, result, msg, ts, serialize_out=serial_through)
 
 
-def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False, logger=logger):
+def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False, logger=None):
     """Save products and returns URNs.
 
     Saving Products to HTTPpool will have data stored on the server side. The server only returns URN strings as a response. ProductRefs will be generated by the associated httpclient pool which is the front-end on the user side.
@@ -302,6 +311,9 @@ def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False,
     :tags: a list of tag strings. default is None meaning no tag.
     Returns a URN object or a list of URN objects.
     """
+    if not logger:
+        logger = current_app.logger
+
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
 
     poolname = paths[0]
@@ -313,7 +325,8 @@ def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False,
     if checkpath(fullpoolpath, current_app.config['PC']['self_username']) is None:
         result = FAILED
         msg = 'Pool directory error: ' + fullpoolpath
-        logger.info(f'SAVE product to {poolurl} {msg}.')
+        if logger.isEnabledFor(logging_INFO):
+            logger.info(f'SAVE product to {poolurl} {msg}.')
         return 400, result, msg
 
     # logger.debug(str(id(PM_S._GlobalPoolList)) + ' ' + str(PM_S._GlobalPoolList))
@@ -326,10 +339,12 @@ def save_product(data, paths, tags=None, serialize_in=True, serialize_out=False,
         urn = result.split(poolname, 1)[1][1:-1] if serialize_out else result
         msg = f'Save {urn} to {poolurl} OK.'
         code = 200
-        logger.info(msg)
+        if logger.isEnabledFor(logging_INFO):
+            logger.info(msg)
     except Exception as e:
         code, result, msg = excp(e, serialize_out=serialize_out)
-        logger.info(f'SAVE product to {poolurl} {msg}.')
+        if logger.isEnabledFor(logging_INFO):
+            logger.info(f'SAVE product to {poolurl} {msg}.')
     return code, result, msg
 
 ######################################
@@ -354,8 +369,9 @@ def data_paths(pool, data_paths):
 
     paths = [pool] + parts2paths(data_paths)
 
-    logger.debug('>>>[%4s] %s data_paths= %s paths= %s' %
-                 (request.method, pool, str(data_paths), str(paths)))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('>>>[%4s] %s data_paths= %s paths= %s' %
+                     (request.method, pool, str(data_paths), str(paths)))
 
     code, result, msg = getProduct_Or_Component(
         paths, serialize_out=serial_through)
@@ -367,9 +383,11 @@ def getProduct_Or_Component(paths, serialize_out=False):
     :serialize_out: see :meth:`ProductPool.saveProduct`
     """
 
+    logger = current_app.logger
     lp = len(paths)
     # now paths = poolname, prod_type , ...
-    logger.debug('get prod or compo: ' + str(paths))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('get prod or compo: ' + str(paths))
 
     ts = time.time()
     mInfo = 0
@@ -381,7 +399,8 @@ def getProduct_Or_Component(paths, serialize_out=False):
         if len(mp) < 2:
             msg = 'Need a dot-separated full type name, not %s.' % pp
             code = 422
-            logger.info(f'{code} {msg}')
+            if logger.isEnabledFor(logging_INFO):
+                logger.info(f'{code} {msg}')
             return code, '"FAILED"', msg
         modname, ptype = mp[0], mp[1]
         cls = Classes.mapping[ptype]
@@ -393,7 +412,8 @@ def getProduct_Or_Component(paths, serialize_out=False):
         # non-serialized
         code = 200
         msg = 'Getting API info for %s OK' % paths[1]
-        logger.info(f'{code} {msg}')
+        if logger.isEnabledFor(logging_INFO):
+            logger.info(f'{code} {msg}')
         return 0, resp(code, mInfo,  msg, ts, serialize_out=False), 0
     # elif lp == 3 and paths[-1]=='':
 
@@ -407,11 +427,10 @@ def getProduct_Or_Component(paths, serialize_out=False):
         return get_component_or_method(paths, mInfo, serialize_out=serialize_out)
 
     else:
-        __import__("pdb").set_trace()
-
         code = 400
         msg = 'Unknown path %s' % str(paths)
-        logger.info(f'{code} {msg}')
+        if logger.isEnabledFor(logging_INFO):
+            logger.info(f'{code} {msg}')
         return code, '"FAILED"', msg
 
 
@@ -435,8 +454,11 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
     Except for full products, most components  are not in serialized form.
     """
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
+    logger = current_app.logger
+
     ts = time.time()
-    logger.debug('get compo or meth: ' + str(paths))
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('get compo or meth: ' + str(paths))
     lp = len(paths)
     # if paths[-1] in ('toString', 'string'):
     #    __import__('pdb').set_trace()
@@ -536,6 +558,7 @@ def load_component_at(pos, paths, mInfo):
     # component = fetch(paths[pos+2:], mInfo)
     # if component:
 
+    logger = current_app.logger
     # get the product live
     code, live_prod, msg = load_product(pos, paths, serialize_out=False)
     if code != 200:
@@ -551,6 +574,8 @@ def load_product(p, paths, serialize_out=False):
     """
     FAILED = '"FAILED"' if serialize_out else 'FAILED'
 
+    logger = current_app.logger
+
     typename = paths[p]
     indexstr = paths[p+1]
     poolname = paths[0]
@@ -558,7 +583,8 @@ def load_product(p, paths, serialize_out=False):
     urn = makeUrn(poolname=poolname, typename=typename, index=indexstr)
     # resourcetype = fullname(data)
 
-    logger.debug('LOAD product: ' + urn)
+    if logger.isEnabledFor(logging_DEBUG):
+        logger.debug('LOAD product: ' + urn)
     try:
         poolobj = PM_S.getPool(poolname=poolname, poolurl=poolurl)
         result = poolobj.loadProduct(urn=urn, serialize_out=serialize_out)
