@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from .taggable import Taggable
 from .urn import Urn, parseUrn, makeUrn
-from fdi.dataset.odict import ODict
+from .productpool import ProductPool
 
 import logging
 # create logger
@@ -9,16 +9,98 @@ logger = logging.getLogger(__name__)
 # logger.debug('level %d' %  (logger.getEffectiveLevel()))
 
 # List of Housekeeping DBs
+# HKDBS = ['classes', 'tags', 'urns', 'dTypes', 'dTags']
 # some new ####
-HKDBS = ['classes', 'tags', 'urns', 'dTypes', 'dTags']
+HKDBS = ['dTypes', 'dTags']
+
+""" Reference Data Model for ManagedPool.
+
+The Housekeeping Tables v2.0: `dTypes` and `dTags`
+--------------------------------------------------
+
+The FDI `ProductPool` stors data and their references.
+The products are organized using their Data Type name, e.g. `fdi.dataset.product.Product`, in the Housekeeping table named `dTypes`. 
+
+When initialized, the `dTypes` table is empty mapping. When a data item is saved in the pool, `dTypes` gets a key named by the Data Type, with a mapping (DTM) as the key's value to hold other information of the data. Every DTM has two entries: 'sn' and 'currentSN'.
+
+Every data item of a certain Data Type is given a serial number (sn) to index identify it within the DMT, which uses the serial numbers as the key and a mapping (SNM) to hold information of this product: `tags` list, optionally `meta` for metadata access, and `refcnt` for refereced state. The latest allocated serial number is recorded with the 'currentSN' key in the DTM. Every data item added to the pool increments 'currentSN' of the data type by 1.
+
+When a data item is removed, its serial number, with its SNM, are removed from its associated DTM.
+
+All data are cleared by initialization. Removing all items within a DTM does not reset its `currentSN`. For example if you::
+
+   * add a Product of a new type, you will get ``sn=0, currentSN=0``.
+   * remove it and the size of the SNM becomes 0 and `currentSN` stay the same.
+   * add it again, you will get ``sn=1, currentSN=1``.
+
+`dTypes` schematics::
+
+.. code-block:: 
+
+                     $Datatype_Name0:
+                            'currentSN': $csn
+                            'sn':
+                                $sn0:
+                                   tags: {$tag1, $tag2, ...
+                                   meta: [$start, $end]
+                                   refcnt: $count
+                                $sn1:
+                                   tags: {$tag1, $tag2, ...
+                                   meta: [$start, $end]
+                                   refcnt: $count
+
+               example::
+
+                    'foo.bar.Bar':
+                            'currentSN': 1
+                            'sn':
+                                0:
+                                   'tags': {'cat', 'white'}
+                                   'meta': [123, 456]
+                                   'refcnt': 0
+                                1:
+                                   'tags': {'dog', 'white'}
+                                   'meta': [321, 765]
+                                   'refcnt': 0
+                    'foo.baz.Baz':
+                            'currentSN': 34
+                            'sn':
+                                34:
+                                   'tags': {'tree', 'green'}
+                                   'meta': [100, 654]
+                                   'refcnt': 1
+
+The `dTags` table is the other table of the Pool.  Itdiffers from `tags` in v1:;
+
+    1. uses `dTypes:[sn]`, instead of `urn`, so there is no poolname anywhere,
+    2. simplify by removing second level dict.
+
+When a data item is saved with one or multiple tag strings. The key is the tag (iterate if there are more than one tag), the value is the Tags MApping (TM). The Data Type is the key in the TM, the value is a list sn number from filling the `dTypes` table is appended to the list. When this data item is removed, so is its entry in the list. If the list is empty, the Data Type is removed from `dTags`.
+
+The schematic of the `dTags` table is::
+
+.. code::
+
+                     $tag_name0:
+                           $Datatype_Name1:{$sn1, $sn2...}
+                           $Datatype_Name2:{$sn3, ...}
+
+example::
+
+                     'cat': { 'foo.bar.Bar':{0} }
+                     'white': { 'foo.bar.Bar'; {0, 1} }
+                     'dog': ...
+"""
 
 
-def get_missing(self, urn, datatype, sn, no_check=False):
+def get_missing(self, urn, datatype, sn, no_check=False,
+                int_index=True):
     """ make URN(s) if datatype and sn(s) are given and vice versa.
 
     Parameters
     ----------
-
+    int_index : bool
+       Set `True` to return integer `sn` (Default)
     no_check: bool
         Do not Check if `datatype` and `sn` are in the pool's HK.
         Default is `False`
@@ -39,7 +121,7 @@ def get_missing(self, urn, datatype, sn, no_check=False):
 
     if datatype is None or sn is None and urn is not None:
         # new ###
-        poolname, datatype, sn = parseUrn(urn, int_index=True)
+        poolname, datatype, sn = parseUrn(urn, int_index=int_index)
     else:
         # datatype+sn takes priority over urn
         urn = makeUrn(self._poolname, datatype, sn)
@@ -53,10 +135,10 @@ def get_missing(self, urn, datatype, sn, no_check=False):
             dat = [datatype]
             sns = [sn]
             for d, s in zip(dat, sns):
-                if d not in self._dTypes:
+                if hasattr(self, 'serverDatatypes') and d not in self.serverDatatypes:
                     raise KeyError(
-                        f'{d} not found in pool {self._poolname}')
-                if s not in self._dTypes[d]['sn']:
+                        f'{d} not found on server{self._poolname}')
+                if d not in self._dTypes or s not in self._dTypes[d]['sn']:
                     raise IndexError('%s:%d not found in pool %s.' %
                                      (d, s, self._poolname))
     # /new ###
@@ -85,41 +167,124 @@ def add_tag_datatype_sn(tag, datatype, sn, dTypes=None, dTags=None):
     """
     if not tag:
         return
-    snt = dTypes[datatype]['sn'][sn]['tags']
-    if tag not in snt:
-        snt.append(tag)
+
+    if not isinstance(sn, int):
+        raise TypeError('serial number must be an integer.')
+    snt = dTypes[datatype]['sn'][sn].get('tags', set())
+    if isinstance(snt, list):
+        snt = set(snt)
+    # if tag not in snt:
+    snt.add(tag)
+    dTypes[datatype]['sn'][sn]['tags'] = snt  # newly created {tag..}
     # dTags saves datatype:sn
     typ = datatype
     if tag not in dTags:
         dTags[tag] = {}
     t = dTags[tag]
     if typ not in t:
-        t[typ] = [str(sn)]
+        t[typ] = {str(sn)}
     else:
-        t[typ].append(str(sn))
+        if isinstance(t[typ], list):
+            t[typ] = set(t[typ])
+        t[typ].add(str(sn))
 
 
-class DictHk(Taggable):
+def populate_pool2(tags, ptype, sn=None, dTypes=None, dTags=None):
+    """Add a new product to Housekeeping Tables v2.
+
+    A new product is representated by its type name and optional
+    serial number (aka Index), and tags.
+
+    Parameters
+    ----------
+    tags : list
+        The tags in a list. `None` and empty tags are ignored.
+    ptype : str
+        The product name / datatype / class name of the data item, new or existing.
+    sn : str
+        Serial number. If is `None`, it is assigned as the one in `dTypes`.
+
+    Returns
+    -------
+    tuple
+        dTypes and dTags with updates, and the index/serial number
+    """
+
+    # new ###
+    if dTypes is None:
+        dTypes = {}
+    if dTags is None:
+        dTags = {}
+
+    if ptype in dTypes:
+        if sn is None:
+            int_sn = dTypes[ptype]['currentSN'] + 1
+        else:
+            int_sn = int(sn)
+    else:
+        int_sn = 0 if sn is None else int(sn)
+        dTypes[ptype] = {
+            'currentSN': int_sn,
+            'sn': {}
+        }
+
+    snd = dTypes[ptype]['sn']
+    _sn = str(int_sn)
+    if int_sn not in snd:
+        snd[int_sn] = {
+        }
+
+    dTypes[ptype]['currentSN'] = int_sn
+
+    # /new #####
+    if tags is not None:
+        for t in tags:
+            add_tag_datatype_sn(t, ptype, int_sn, dTypes, dTags)
+
+    return dTypes, dTags, _sn
+
+
+class DictHk(ProductPool):
     """
     Definition of services provided by a product storage supporting versioning.
     """
 
     def __init__(self, **kwds):
-        super(DictHk, self).__init__(**kwds)
-        # {tag->{'urns':[urn]}
-        self._tags = dict()
-        # {urn->{'tags':[tag], 'meta':meta}}
-        self._urns = dict()
-        # new ###
-        self._dTypes = dict()
+        super().__init__(**kwds)
+        # self._dTypes = dict()
+
+    def setup(self):
+        """ Sets up interal machiney of this Pool,
+        but only if self._poolname and self._poolurl are present,
+        and other pre-requisits are met.
+
+        Subclasses should implement own setup(), and
+        make sure that self._poolname and self._poolurl are present with ``
+
+        if <pre-requisit not met>: return True
+        if super().setup(): return True
+
+        # super().setup() has done its things by now.
+        <do setup>
+        return False
+``
+        returns: True if not both  self._poolname and self._poolurl are present.
+
+        """
+
+        if super().setup():
+            return True
+        # new ##
+
         self._dTags = dict()
 
-    def getTags(self, urn=None, datatype=None, sn=None):
+        return False
+
+    def getTags(self, urn=None, datatype=None, sn=None, **kwds):
         """ 
         Get all of the tags that map to a given URN or a pair of data type and serial number.
 
         Get all known tags if input arenot specified.
-        mh: returns an iterator.
 
         If datatype and sn are given, use them and ignore urn.
         """
@@ -130,11 +295,12 @@ class DictHk(Taggable):
             return self._dTags.keys()
 
         # new ###
-        if 0:
-            assert self._urns[urn]['tags'] == self._dTypes[datatype]['sn'][sn]['tags']
-            return self._urns[urn]['tags']
+        # assert self._urns[urn]['tags'] == self._dTypes[datatype]['sn'][sn]['tags']
+        # return self._urns[urn]['tags']
 
-        return self._dTypes[datatype]['sn'][sn]['tags']
+        p = self._dTypes[datatype]['sn'][sn].get('tags', set())
+
+        return p
 
     get_missing = get_missing
 
@@ -142,14 +308,11 @@ class DictHk(Taggable):
         """
         Get the full tag->urn mappings.
 
-        mh: returns an iterator
-        csdb: csdb/v1/storage/tag?tag=tag1,tag2
         """
         # new ###
         return self._dTags
 
-        if 0:
-            return zip(self._tags.keys(), map(lambda v: v['urns'], self._value()))
+        # return zip(self._tags.keys(), map(lambda v: v['urns'], self._value()))
 
     def getUrn(self, tag):
         """
@@ -210,29 +373,38 @@ class DictHk(Taggable):
                 if len(cross_ref_map[val]) == 0:
                     cross_ref_map.pop(val)
 
-    def removeTag(self, tag):
+    def removeTag(self, tag, **kwds):
         """
-        Remove the given tag from the tag and urn maps.
-        # TODO in CSDB
+        Remove the given tag from the H/K maps.
+
         """
+
         # new ##
+        if tag not in self._dTags:
+            logger.debug('Tag "{tag}" not found in pool {self._poolname}.')
+            return 0
         clsn_sns = self._dTags.pop(tag)
+        # {datatype:[sn0, sn1..]}
         for datatype, sns in clsn_sns.items():
             for sn in sns:
+                # clear the tag from dTypes
                 sn = int(sn)
                 ts = self._dTypes[datatype]['sn'][sn]['tags']
                 if tag in ts:
                     ts.remove(tag)
-                    if len(tags) == 0:
-                        del ts
                 else:
                     logger.warning('tag %s missing from %s:%s:%s.' %
                                    (tag, self._poolname, datatype, sn))
-        if 0:
-            self.removekey(tag, self._tags, 'tags', self._urns, 'urns')
+                # Do not remove in the for sn .. loop, or the next tag
+                # may complain no sn
+                if len(ts) == 0:
+                    del ts
+        # self.removekey(tag, self._tags, 'tags', self._urns, 'urns')
         # new ##
         # assert list(self._tags) == list(self._dTags)
 
+        return 0
+
     def removeUrn(self, urn=None, datatype=None, sn=None):
         """
         Remove the given urn (or a pair of data type and serial number) from the tag and urn maps.
@@ -241,16 +413,18 @@ class DictHk(Taggable):
         """
         u, datatype, sn = self.get_missing(
             urn=urn, datatype=datatype, sn=sn,
-            no_check=False)
+            no_check=True)
         # new ##
         from .productpool import ProductPool
         dats, sns, alist = ProductPool.vectorize(datatype, sn)
         for d, s in zip(dats, sns):
-            # if d not in self._dTypes:
-            #     if self.ignore_error_when_delete:
-            #         return -1
-            #     else:
-            #         raise ValueError(f'{d} not found in {self.poolname}.')
+            if not hasattr(self, '_dTypes') or d not in self._dTypes:
+                msg = f'{d} not found on server.'
+                if not self.ignore_error_when_delete:
+                    raise ValueError(msg)
+                else:
+                    logger.debug(msg)
+
             _snd = self._dTypes[d]['sn']
             if s not in _snd:
                 msg = f'{s} not found in pool {self.getId()}.'
@@ -262,6 +436,7 @@ class DictHk(Taggable):
             if 'tags' in _snd[s]:
                 for tag in _snd[s]['tags']:
                     if tag in self._dTags:
+                        # remove sn from datatype
                         self._dTags[tag][d].remove(str(s))
                         if len(self._dTags[tag][d]) == 0:
                             del self._dTags[tag][d]
@@ -276,11 +451,10 @@ class DictHk(Taggable):
                                        (tag, self._poolname, d, s))
             _snd.pop(s)
             if len(_snd) == 0:
-                del self._dTypes[d]
+                pass  # del self._dTypes[d]
             # /new ##
 
-            if 0:
-                self.removekey(u, self._urns, 'urns', self._tags, 'tags')
+            # self.removekey(u, self._urns, 'urns', self._tags, 'tags')
             # new ##
             # assert s not in self._dTypes[d]['sn']
 
@@ -288,7 +462,6 @@ class DictHk(Taggable):
         """
         Sets the specified tag to the given URN or a pair of data type and serial number.
 
-        # TODO in CSDB
         """
         u, datatype, sn = self.get_missing(
             urn=urn, datatype=datatype, sn=sn, no_check=True)
